"use strict";

// Import dependencies
import("/js/core.js").then(() =>
	import("/js/damper.js").then(() =>
		import("/js/loader.js").then(() =>
			import("/js/telegram.js").then(() =>
				import("/js/hotline.js").then(() => {
					const dependencies = setInterval(() => {
						if (
							typeof core === "function" &&
							typeof core.damper === "function" &&
							typeof core.telegram === "function" &&
							typeof core.hotline === "function"
						) {
							clearInterval(dependencies);
							clearTimeout(timeout);
							initialization();
						}
					}, 10);
					const timeout = setTimeout(() => {
						clearInterval(dependencies);
						initialization();
					}, 5000);

					function initialization() {
						if (typeof core.catalog === "undefined") {
							// Not initialized

							/**
							 * @name Catalog
							 *
							 * @description
							 * Implements actions with catalog
							 *
							 * @memberof core
							 *
							 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
							 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
							 */
							core.catalog = class catalog {
								// Title
								static element_title = core.main.getElementsByTagName("h2")[0];

								// Search search <search>
								static element_search = document.getElementById("search");

								// Search <input>
								static element_search_input =
									this.element_search.getElementsByTagName("input")[0];

								// Categories <section>
								static element_categories =
									document.getElementById("categories");

								// Filters <section>
								static element_filters = document.getElementById("filters");

								// Sorting <section>
								static element_sorting = document.getElementById("sorting");

								// Products <section>
								static element_products = document.getElementById("products");

								/**
								 * Parameters of search
								 *
								 * Will be converted to the body of the GET request ("?name=value&name2=value2")
								 */
								static parameters = new Map([
									[
										"text",
										new URLSearchParams(document.location.search).get("text"),
									],
									[
										"category",
										new URLSearchParams(document.location.search).get(
											"category"
										),
									],
									[
										"sort",
										new URLSearchParams(document.location.search).get("sort"),
									],
									[
										"brand",
										new URLSearchParams(document.location.search).get("brand"),
									],
								]);

								/**
								 * Search (interface)
								 *
								 * Request search in the catalog and render in the core.main
								 *
								 * @param {Event} event Event (keyup)
								 * @param {bool} force Ignore the damper? (false)
								 *
								 * @return {bool} True if an error occurs to continue the event execution
								 */
								static search(event, force = false) {
									if (this.element_search_input instanceof HTMLElement) {
										// Found the search <input> element

										// Removing errors animations
										this.element_search_input.classList.remove("error");

										if (
											this.element_search_input.value.length < 3 &&
											this.element_search_input.value.lenght > 0
										) {
											// Only 1-2 character entered (this is not a search reset and is not an adequate search)

											// Exit (fail)
											return true;
										}
									}

									if (event instanceof Event) {
										// Executed from event

										if (event.type === "keyup") {
											// Executed by "keyup" event

											if (event.target === this.element_search_input) {
												// Executed from search <input> element

												if (event.keyCode === 13) {
													// Executed by "enter" button

													// Execute
													this.search_system();
												} else if (this.element_search_input.value.length < 3) {
													// Less than 3 character entered (this is not a search reset and is not an adequate search)

													// Exit (fail)
													return true;
												} else {
													// Executed by any else button

													// Execute under damper
													this.search_damper(force);
												}
											}
										} else {
											// Executed not by "keyup" event

											// Execute under damper
											this.search_damper(force);
										}

										// Exit (success)
										return false;
									}

									// Exit (fail)
									return true;
								}

								/**
								 * Search (damper)
								 *
								 * Request search in the catalog and render in the core.main
								 *
								 * @param {bool} force Ignore the damper? (false)
								 *
								 * @return {void}
								 */
								static search_damper = core.damper(
									(...variables) => this.search_system(...variables),
									1400,
									1
								);

								/**
								 * Search (system)
								 *
								 * Request search in the catalog and render in the core.main
								 *
								 * @return {Promise} Request to the server
								 *
								 * @todo add animations of errors
								 */
								static async search_system() {
									// Blocking the search <input> element
									this.element_search_input.setAttribute("disabled", true);

									// @todo add timeout to unblock

									// Initialize the buffer of URN parameters
									const parameters = new URLSearchParams(
										window.location.search
									);

									// Iterate parameters
									this.parameters.forEach((value, parameter) => {
										if (
											(typeof value === "string" ||
												typeof value === "number") &&
											!(
												(parameter === "text" && value === null) ||
												value.length < 3
											)
										) {
											// Validated value

											// Write parameter to the buffer of URN parameters
											parameters.set(parameter, value);
										} else parameters.delete(parameter);
									});

									return await core.loader
										.load("?" + parameters)
										.then((json) => {
											if (json) {
												// Received response

												this.element_search_input.removeAttribute("disabled");
												// element.focus();

												/**
												 * Title <h2>
												 */
												if (
													typeof json.title === "string" &&
													json.title.length > 0
												) {
													// Received and validated text of the title <h2>

													if (this.element_title instanceof HTMLElement) {
														// Found the title <h2> element

														// Writing into the title <h2> element
														this.element_title.innerText = json.title;
													} else {
														// Not found the title <h2> element

														// Initialize the title <h2> element
														this.element_title = document.createElement("h2");

														if (core.main instanceof HTMLElement) {
															// Found the <main> element

															// Inititalize the first element in the <main> element
															const first = core.main.firstElementChild;

															if (first instanceof HTMLElement) {
																// Initialized the first element in the <main> element

																// Writing the title <h2> element before the first element in the <main> element
																core.main.insertBefore(
																	this.element_title,
																	first
																);
															}
														}

														// Writing into the title <h2> element
														this.element_title.innerText = json.title;
													}
												} else {
													// Not received or not validated text of the title <h2>

													if (this.element_title instanceof HTMLElement) {
														// Initialized the title <h2> element

														if (json.title === "") {
															// Requested to delete the title <h2> element

															// Deleting the title <h2> element
															this.element_title.remove();
														}
													}
												}

												/**
												 * Search search <search>
												 */
												if (
													typeof json.search === "string" &&
													json.search.length > 0
												) {
													// Received and validated the searcn HTMl-code

													if (this.element_search instanceof HTMLElement) {
														// Found the search <search> element

														// Writing into the search <search> element
														this.element_search.outerHTML = json.search;

														// Reinitializing the parameter with the search <search> element
														this.element_search =
															document.getElementById("search");
													} else {
														// Not found the search <search> element

														// Initialize the search <search> element
														this.element_search =
															document.createElement("search");

														if (core.main instanceof HTMLElement) {
															// Found the <main> element

															if (this.element_title instanceof HTMLElement) {
																// Initialized the title <h2> elemment in the <main> element

																// Writing the search <search> element after the title <h2> element in the <main> element
																core.main.insertBefore(
																	this.element_search,
																	this.element_title.nextSibling
																);
															} else {
																// Not initialized the title <h2> element in the <main> element

																// Inititalize the first element in the <main> element
																const first = core.main.firstElementChild;

																if (first instanceof HTMLElement) {
																	// Initialized the first element in the <main> element

																	// Writing the search <search> element before the first element in the <main> element
																	core.main.insertBefore(
																		this.element_search,
																		first
																	);
																}
															}

															// Writing into the search <search> element
															this.element_search.outerHTML = json.search;

															// Reinitializing the parameter with the search <search> element
															this.element_search =
																document.getElementById("search");

															// Reinitializing the parameter with the search <input> element
															this.element_search_input =
																this.element_search.getElementsByTagName(
																	"input"
																)[0];
														}
													}
												} else {
													// Not received or not validated the search HTML-code

													if (this.element_search instanceof HTMLElement) {
														// Initialized the search <search> element

														if (json.search === "") {
															// Requested to delete the search <search> element

															// Deleting the search <search> element
															this.element_search.remove();
														}
													}
												}

												/**
												 * Categories
												 */
												if (
													typeof json.categories === "string" &&
													json.categories.length > 0
												) {
													// Received and validated the categories HTMl-code

													if (this.element_categories instanceof HTMLElement) {
														// Found the categories <section> element

														// Writing into the categories <section> element
														this.element_categories.outerHTML = json.categories;

														// Reinitializing the parameter with the categories <section> element
														this.element_categories =
															document.getElementById("categories");
													} else {
														// Not found the categories <section> element

														// Initialize the categories <section> element
														this.element_categories =
															document.createElement("section");

														if (core.main instanceof HTMLElement) {
															// Found the <main> element

															if (this.element_search instanceof HTMLElement) {
																// Initialized the search <search> element in the <main> element

																// Writing the categories <section> element after the search <search> element in the <main> element
																core.main.insertBefore(
																	this.element_categories,
																	this.element_search.nextSibling
																);
															} else if (
																this.element_title instanceof HTMLElement
															) {
																// Initialized the title <h2> element in the <main> element

																// Writing the categories <section> element after the title <h2> element in the <main> element
																core.main.insertBefore(
																	this.element_categories,
																	this.element_title.nextSibling
																);
															} else {
																// Not initialized the title <h2> element in the <main> element

																// Inititalize the first element in the <main> element
																const first = core.main.firstElementChild;

																if (first instanceof HTMLElement) {
																	// Initialized the first element in the <main> element

																	// Writing the categories <section> before the first element in the <main> element
																	core.main.insertBefore(
																		this.element_categories,
																		first
																	);
																}
															}

															// Writing into the categories <section> element
															this.element_categories.outerHTML =
																json.categories;

															// Reinitializing the parameter with the categories <section> element
															this.element_categories =
																document.getElementById("categories");
														}
													}
												} else {
													// Not received or not validated the categories HTML-code

													if (this.element_categories instanceof HTMLElement) {
														// Initialized the categories <section> element

														// Deleting the categories <section> element
														this.element_categories.remove();
													}
												}

												/**
												 * Filters
												 */
												if (
													typeof json.filters === "string" &&
													json.filters.length > 0
												) {
													// Received and validated the filters HTMl-code

													if (this.element_filters instanceof HTMLElement) {
														// Found the filters <section> element

														// Writing into the filters <section> element
														this.element_filters.outerHTML = json.filters;

														// Reinitializing the parameter with the filters <section> element
														this.element_filters =
															document.getElementById("filters");
													} else {
														// Not found the filters <section> element

														// Initialize the filters <section> element
														this.element_filters =
															document.createElement("section");

														if (core.main instanceof HTMLElement) {
															// Found the <main> element

															if (
																this.element_categories instanceof HTMLElement
															) {
																// Initialized the categories <section> element in the <main> element

																// Writing the filters <section> element after the categories <section> element in the <main> element
																core.main.insertBefore(
																	this.element_filters,
																	this.element_categories.nextSibling
																);
															} else if (
																this.element_search instanceof HTMLElement
															) {
																// Initialized the search <search> element in the <main> element

																// Writing the filters <section> element after the search <search> element in the <main> element
																core.main.insertBefore(
																	this.element_filters,
																	this.element_search.nextSibling
																);
															} else if (
																this.element_title instanceof HTMLElement
															) {
																// Initialized the title <h2> element in the <main> element

																// Writing the filters <section> element after the title <h2> element in the <main> element
																core.main.insertBefore(
																	this.element_filters,
																	this.element_title.nextSibling
																);
															} else {
																// Not initialized the title <h2> element in the <main> element

																// Inititalize the first element in the <main> element
																const first = core.main.firstElementChild;

																if (first instanceof HTMLElement) {
																	// Initialized the first element in the <main> element

																	// Writing the filters <section> before the first element in the <main> element
																	core.main.insertBefore(
																		this.element_filters,
																		first
																	);
																}
															}

															// Writing into the filters <section> element
															this.element_filters.outerHTML = json.filters;

															// Reinitializing the parameter with the filters <section> element
															this.element_filters =
																document.getElementById("filters");
														}
													}
												} else {
													// Not received or not validated the filters HTML-code

													if (this.element_filters instanceof HTMLElement) {
														// Initialized the filters <section> element

														// Deleting the filters <section> element
														this.element_filters.remove();
													}
												}

												/**
												 * Sorting
												 */
												if (
													typeof json.sorting === "string" &&
													json.sorting.length > 0
												) {
													// Received and validated the sorting HTMl-code

													if (this.element_sorting instanceof HTMLElement) {
														// Found the sorting <section> element

														// Writing into the sorting <section> element
														this.element_sorting.outerHTML = json.sorting;

														// Reinitializing the parameter with the sorting <section> element
														this.element_sorting =
															document.getElementById("sorting");
													} else {
														// Not found the sorting <section> element

														// Initialize the sorting <section> element
														this.element_sorting =
															document.createElement("section");

														if (core.main instanceof HTMLElement) {
															// Found the <main> element

															if (this.element_filters instanceof HTMLElement) {
																// Initialized the filters <section> element in the <main> element

																// Writing the sorting <section> element after the filters <section> element in the <main> element
																core.main.insertBefore(
																	this.element_sorting,
																	this.element_filters.nextSibling
																);
															} else if (
																this.element_categories instanceof HTMLElement
															) {
																// Initialized the categories <section> element in the <main> element

																// Writing the sorting <section> element after the categories <section> element in the <main> element
																core.main.insertBefore(
																	this.element_sorting,
																	this.element_categories.nextSibling
																);
															} else if (
																this.element_search instanceof HTMLElement
															) {
																// Initialized the search <search> element in the <main> element

																// Writing the sorting <section> element after the search <search> element in the <main> element
																core.main.insertBefore(
																	this.element_sorting,
																	this.element_search.nextSibling
																);
															} else if (
																this.element_title instanceof HTMLElement
															) {
																// Initialized the title <h2> element in the <main> element

																// Writing the sorting <section> element after the title <h2> element in the <main> element
																core.main.insertBefore(
																	this.element_sorting,
																	this.element_title.nextSibling
																);
															} else {
																// Not initialized the title <h2> element in the <main> element

																// Inititalize the first element in the <main> element
																const first = core.main.firstElementChild;

																if (first instanceof HTMLElement) {
																	// Initialized the first element in the <main> element

																	// Writing the sorting <section> before the first element in the <main> element
																	core.main.insertBefore(
																		this.element_sorting,
																		first
																	);
																}
															}

															// Writing into the sorting <section> element
															this.element_sorting.outerHTML = json.sorting;

															// Reinitializing the parameter with the sorting <section> element
															this.element_sorting =
																document.getElementById("sorting");
														}
													}
												} else {
													// Not received or not validated the sorting HTML-code

													if (this.element_sorting instanceof HTMLElement) {
														// Initialized the sorting <section> element

														// Deleting the sorting <section> element
														this.element_sorting.remove();
													}
												}

												/**
												 * Products
												 */
												if (
													typeof json.products === "string" &&
													json.products.length > 0
												) {
													// Received and validated the products HTMl-code

													if (this.element_products instanceof HTMLElement) {
														// Found the products <section> element

														// Writing into the products <section> element
														this.element_products.outerHTML = json.products;

														// Reinitializing the parameter with the products <section> element
														this.element_products =
															document.getElementById("products");
													} else {
														// Not found the products <section> element element

														// Initialize the products <section> element
														this.element_products =
															document.createElement("section");

														if (core.main instanceof HTMLElement) {
															// Found the <main> element

															if (this.element_sorting instanceof HTMLElement) {
																// Initialized the sorting <section> element in the <main> element

																// Writing the products <section> element after the sorting <section> element in the <main> element
																core.main.insertBefore(
																	this.element_products,
																	this.element_sorting.nextSibling
																);
															} else if (
																this.element_filters instanceof HTMLElement
															) {
																// Initialized the filters <section> element in the <main> element

																// Writing the products <section> element after the filters <section> element in the <main> element
																core.main.insertBefore(
																	this.element_products,
																	this.element_filters.nextSibling
																);
															} else if (
																this.element_categories instanceof HTMLElement
															) {
																// Initialized the categories <section> element in the <main> element

																// Writing the products <section> element after the categories <section> element in the <main> element
																core.main.insertBefore(
																	this.element_products,
																	this.element_categories.nextSibling
																);
															} else if (
																this.element_search instanceof HTMLElement
															) {
																// Initialized the search <search> element in the <main> element

																// Writing the products <section> element after the search <search> element in the <main> element
																core.main.insertBefore(
																	this.element_products,
																	this.element_search.nextSibling
																);
															} else if (
																this.element_title instanceof HTMLElement
															) {
																// Initialized the title <h2> element in the <main> element

																// Writing the products <section> element after the title <h2> element in the <main> element
																core.main.insertBefore(
																	this.element_products,
																	this.element_title.nextSibling
																);
															} else {
																// Not initialized the title <h2> element in the <main> element

																// Inititalize the first element in the <main> element
																const first = core.main.firstElementChild;

																if (first instanceof HTMLElement) {
																	// Initialized the first element in the <main> element

																	// Writing the products <section> before the first element in the <main> element
																	core.main.insertBefore(
																		this.element_products,
																		first
																	);
																}
															}

															// Writing into the products <section> element
															this.element_products.outerHTML = json.products;

															// Reinitializing the parameter with the products <section> element
															this.element_products =
																document.getElementById("products");
														}
													}
												} else {
													// Not received or not validated the products HTML-code

													if (this.element_products instanceof HTMLElement) {
														// Initialized the products <section> element

														// Deleting the products <section> element
														this.element_products.remove();
													}
												}
											}
										});
								}

								/**
								 * Product card (interface)
								 *
								 * Request product data and render in core.window
								 *
								 * @param {HTMLElement} button Button of the product <a>
								 * @param {bool} force Ignore the damper? (false)
								 *
								 * @return {bool} True if an error occurs to continue the event execution
								 */
								static product(button, force = false) {
									// Initializing identifier of the category
									const identifier = button.getAttribute(
										"data-product-identifier"
									);

									// Execute under damper
									this.product_damper(identifier, force);

									// Exit (success)
									return false;
								}

								/**
								 * Product card (damper)
								 *
								 * Request product data and render in core.window
								 *
								 * @param {string} identifier Identifier of the product
								 * @param {bool} force Ignore the damper? (false)
								 *
								 * @return {void}
								 */
								static product_damper = core.damper(
									(...variables) => this.product_system(...variables),
									400,
									1
								);

								/**
								 * Product card (system)
								 *
								 * Request product data and render in core.window
								 *
								 * @param {string} identifier Identifier of the product
								 *
								 * @return {Promise} Request to the server
								 */
								static async product_system(identifier) {
									if (typeof identifier === "string") {
										// Validated identifier

										// Initialize the buffer of URN parameters @todo after opening window add to document.location.search
										const parameters = new URLSearchParams();

										// Write parameter to the buffer of URN parameters
										parameters.set("product", identifier);

										return await core.request("?" + parameters).then((json) => {
											if (
												json.errors !== null &&
												typeof json.errors === "object" &&
												json.errors.length > 0
											) {
												// Fail (received errors)
											} else {
												// Success (not received errors)

												if (core.window instanceof HTMLElement) {
													// Initialized active window

													// Delete active window
													core.window.remove();
												}

												if (
													json.product !== null &&
													typeof json.product === "object"
												) {
													// Received data of the product

													// Deinitializing of the old winow
													const old = document.getElementById("window");
													if (old instanceof HTMLElement) old.remove();

													const wrap = document.createElement("section");
													wrap.setAttribute("id", "window");

													const card = document.createElement("div");
													// card.classList.add("product", "card");
													card.classList.add("card", "unselectable");

													const h3 = document.createElement("h3");

													const name = document.createElement("span");
													name.classList.add("name");
													name.setAttribute("title", identifier);
													name.innerText = json.product.name;

													const exit = document.createElement("a");
													exit.classList.add("exit");
													exit.setAttribute("type", "button");

													const exit_icon = document.createElement("i");
													exit_icon.classList.add("icon", "close");

													const images = document.createElement("div");
													images.classList.add("images", "unselectable");

													const button = core.telegram.api.isVisible;

													// блокировка закрытия изображений
													let images_from;
													const _images_from = (event) => (images_from = event);
													images.addEventListener("mousedown", _images_from);
													images.addEventListener("touchstart", _images_from);

													const _open = (event) => {
														if (
															event.type === "touchstart" ||
															event.button === 0
														) {
															{
																let x = event.pageX || event.touches[0].pageX;
																let y = event.pageY || event.touches[0].pageY;
																let _x =
																	images_from.pageX ||
																	images_from.touches[0].pageX;
																let _y =
																	images_from.pageY ||
																	images_from.touches[0].pageY;

																if (
																	_x - x < 10 &&
																	_x - x > -10 &&
																	_y - y < 10 &&
																	_y - y > -10
																) {
																	images.classList.add("extend");

																	if (button) {
																		core.telegram.api.MainButton.hide();
																	}

																	setTimeout(() => {
																		images.hotline.step = 0;
																		images.hotline.wheel = false;
																		images.hotline.start();
																		images.addEventListener("mouseup", _close);
																		images.addEventListener("touchend", _close);
																	}, 300);
																	images.removeEventListener("mouseup", _open);
																	images.removeEventListener("touchend", _open);
																}
															}
														}
													};

													const _close = (event) => {
														let x = event.pageX || event.touches[0].pageX;
														let y = event.pageY || event.touches[0].pageY;
														let _x =
															images_from.pageX || images_from.touches[0].pageX;
														let _y =
															images_from.pageY || images_from.touches[0].pageY;

														if (
															event.type === "touchstart" ||
															event.button === 0
														) {
															if (
																_x - x < 10 &&
																_x - x > -10 &&
																_y - y < 10 &&
																_y - y > -10
															) {
																// Курсор не был сдвинут на 10 квардратных пикселей
																// (в идеале надо переделать на таймер 2 секунды есди зажата кнопка то ничего не делать а просто двигать)
																// пох абсолютно сейчас заказчик слишком охуевший для этого

																images.hotline.step = -0.3;
																images.hotline.wheel = true;

																if (width < card.offsetWidth) {
																	images.hotline.stop();
																}
																images.classList.remove("extend");

																if (button) {
																	core.telegram.api.MainButton.show();
																}

																images.removeEventListener("mouseup", _close);
																images.removeEventListener("touchend", _close);
																images.addEventListener("mousedown", _start);
																images.addEventListener("touchstart", _start);
															}
														}
													};

													const _start = (event) => {
														if (
															event.type === "touchstart" ||
															event.button === 0
														) {
															images.removeEventListener("mousedown", _start);
															images.removeEventListener("touchstart", _start);
															images.addEventListener("mouseup", _open);
															images.addEventListener("touchend", _open);
														}
													};

													images.addEventListener("mousedown", _start);
													images.addEventListener("touchstart", _start);

													for (const uri of json.product.images) {
														const image = document.createElement("img");
														image.setAttribute("src", uri);
														image.setAttribute("ondragstart", "return false;");

														images.append(image);
													}

													const header = document.createElement("p");
													header.classList.add("header");

													const brand = document.createElement("small");
													brand.classList.add("brand");
													brand.innerText = json.product.brand;

													const description = document.createElement("p");
													description.classList.add("description");
													description.innerText = json.product.description;

													const compatibility = document.createElement("p");
													compatibility.classList.add("compatibility");
													compatibility.innerText = json.product.compatibility;

													const footer = document.createElement("div");
													footer.classList.add("footer");
													footer.classList.add("footer");

													const dimensions = document.createElement("small");
													dimensions.classList.add("dimensions");

													const x = json.product.dimensions.x;
													const y = json.product.dimensions.y;
													const z = json.product.dimensions.z;

													let formatted = "";

													if (x !== "") formatted = x;
													if (y !== "") {
														if (formatted.length === 0) formatted = y;
														else formatted += "x" + y;
													}
													if (z !== "") {
														if (formatted.length === 0) formatted = z;
														else formatted += "x" + z;
													}

													dimensions.innerText = formatted;

													const weight = document.createElement("small");
													weight.classList.add("weight");
													weight.innerText = json.product.weight + "г";

													const cost = document.createElement("p");
													cost.classList.add("cost");
													cost.innerText = json.product.cost + "р";

													h3.append(name);
													exit.append(exit_icon);
													h3.append(exit);
													card.append(h3);
													card.append(images);
													header.append(brand);
													card.append(header);
													card.append(description);
													card.append(compatibility);
													footer.append(dimensions);
													footer.append(weight);
													footer.append(cost);
													card.append(footer);
													wrap.append(card);
													document.body.append(wrap);

													// Reinitialize parameter
													core.window = document.getElementById('window');

													let width = 0;
													let buffer;
													[...images.children].forEach(
														(child) =>
															(width +=
																child.offsetWidth +
																(isNaN(
																	(buffer = parseFloat(
																		getComputedStyle(child).marginRight
																	))
																)
																	? 0
																	: buffer))
													);

													// блокировка закрытия карточки
													let from;
													const _from = (event) => (from = event.target);
													wrap.addEventListener("mousedown", _from);
													wrap.addEventListener("touchstart", _from);

													const remove = (event) => {
														if (
															typeof event === "undefined" ||
															event.type !== "popstate"
														)
															history.back();

														wrap.remove();
														images.removeEventListener(
															"mousedown",
															_images_from
														);
														images.removeEventListener(
															"touchstart",
															_images_from
														);
														wrap.removeEventListener("mousedown", _from);
														wrap.removeEventListener("touchstart", _from);
														exit.removeEventListener("click", remove);
														exit.removeEventListener("touch", remove);
														document.removeEventListener("click", close);
														document.removeEventListener("touch", close);
														window.removeEventListener("popstate", remove);
													};

													const close = (event) => {
														if (
															from === wrap &&
															!card.contains(event.target) &&
															!!card &&
															!!(
																card.offsetWidth ||
																card.offsetHeight ||
																card.getClientRects().length
															)
														) {
															remove();
														}

														from = undefined;
													};

													exit.addEventListener("click", remove);
													exit.addEventListener("touch", remove);
													document.addEventListener("click", close);
													document.addEventListener("touch", close);
													window.addEventListener("popstate", remove);

													images.hotline = new core.hotline(
														json.product.identfier,
														images
													);
													images.hotline.step = -0.3;
													images.hotline.wheel = true;
													images.hotline.touch = true;

													if (width > card.offsetWidth) {
														images.hotline.start();
													}
												}
											}
										});
									}
								}
							};
						}
					}
				})
			)
		)
	)
);
